/**
 * CRXSocket.cpp
 */
#include "CRXSocket.h"

#include <string.h>

#ifdef _WIN32
# pragma comment (lib, "ws2_32.lib")
# define close(__socket) closesocket(__socket)
typedef int socklen_t;
#endif

bool CRXSocket::mInitialized = false;

CRXSocket::CRXSocket (int aSocket)
	: mSocket (aSocket)
{
	/*----------------------------------------------------------------*/
	memset ((void *)&mAddress, 0x0, sizeof (stSockAddrIn));

	if (!CRXSocket::IsInitialized ())
		CRXSocket::Initialize ();
	/*----------------------------------------------------------------*/
}

CRXSocket::~CRXSocket (void)
{
	/*----------------------------------------------------------------*/
	Close ();
	/*----------------------------------------------------------------*/
}

void
CRXSocket::Attach (int aSocket)
{
	/*----------------------------------------------------------------*/
	mSocket = aSocket;
	/*----------------------------------------------------------------*/
}

int
CRXSocket::Detach (void)
{
	int aSocket = mSocket;

	/*----------------------------------------------------------------*/
	mSocket = 0;

	/*----------------------------------------------------------------*/
	return aSocket;
}

void
CRXSocket::Close (void)
{
	/*----------------------------------------------------------------*/
	if (mSocket <= 0)
		return ;

	close (mSocket);
	mSocket = 0;
	/*----------------------------------------------------------------*/
}

CRXSocket::operator int (void) const
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/
	return mSocket;
}

CRXSocket &
CRXSocket::operator = (int aSocket)
{
	/*----------------------------------------------------------------*/
	Attach (aSocket);
	/*----------------------------------------------------------------*/
	return *this;
}

int
CRXSocket::Initialize (void)
{
	int aResult = 0;

	/*----------------------------------------------------------------*/
	if (CRXSocket::IsInitialized ())
		return aResult;

#ifdef WIN32
	WSADATA aWinSockData;
	
	aResult = WSAStartup (MAKEWORD (2, 0), &aWinSockData);
	if (aResult != 0)
	{
		return ERROR_TCPSOCKET_FAILED_TO_INITIALIZE;
	}
#endif

	mInitialized = true;

	/*----------------------------------------------------------------*/
	return aResult;
}

void
CRXSocket::Finalize (void)
{
	/*----------------------------------------------------------------*/
#ifdef WIN32
	WSACleanup ();
#endif
	/*----------------------------------------------------------------*/
}

bool
CRXSocket::IsInitialized (void)
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/
	return mInitialized;
}

bool
CRXSocket::IsReady (void) const
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/
	return IsInitialized () && IsCreated ();
}

bool
CRXSocket::IsCreated (void) const
{
	/*----------------------------------------------------------------*/
	/*----------------------------------------------------------------*/
	return mSocket > 0 ? true : false;
}

int
CRXSocket::CreateSocket (void)
{
	int aResult = -1;

	/*----------------------------------------------------------------*/
	if (!IsInitialized () || IsCreated ())
		return ERROR_TCPSOCKET_ALREADY_IN_USE;

	aResult = socket (PF_INET, SOCK_STREAM, IPPROTO_TCP);
	if (aResult < 0)
	{
	}

	mSocket = aResult;

	/*----------------------------------------------------------------*/
	return aResult;
}

int
CRXSocket::Connect (std::string	aUrl,
					ushort		aPort)
{
	int				aResult = -1;
	struct hostent	* aHostEnt;

	/*----------------------------------------------------------------*/
	aResult = CreateSocket ();
	if (aResult < 0)
	{
		return aResult;
	}

	mAddress.sin_family			= AF_INET;
	mAddress.sin_port			= htons (aPort);
	mAddress.sin_addr.s_addr	= inet_addr (aUrl.c_str ());

	if (mAddress.sin_addr.s_addr == (unsigned int)-1)
	{
		aHostEnt = gethostbyname (aUrl.c_str ());
		if (aHostEnt == NULL)
		{
			return ERROR_TCPSOCKET_FAILED_TO_GET_HOSTNAME;
		}
		mAddress.sin_family = aHostEnt->h_addrtype;
		memcpy (&(mAddress.sin_addr.s_addr), aHostEnt->h_addr, aHostEnt->h_length);
	}

	aResult = connect (mSocket, (stSockAddr*) &mAddress, sizeof (mAddress));
	if (aResult < 0)
	{
		return ERROR_TCPSOCKET_FAILED_TO_CONNECT;
	}

	/*----------------------------------------------------------------*/
	return aResult;
}

int
CRXSocket::CreateServer (ushort			aPort,
						 int			aBacklog,
						 stSockAddrIn	* aAddress)
{
	int aResult = -1;

	/*----------------------------------------------------------------*/
	aResult = CreateSocket ();
	if (aResult < 0)
	{
		return aResult;
	}

	mAddress.sin_family		= AF_INET;
	mAddress.sin_addr.s_addr= htonl (INADDR_ANY);
	mAddress.sin_port		= htons (aPort);

	aResult = bind (mSocket, (stSockAddr *)&mAddress, sizeof (stSockAddr));
	if (aResult < 0)
	{
		return ERROR_TCPSOCKET_FAILED_TO_BIND;
	}

	aResult = listen (mSocket, aBacklog);
	if (aResult < 0)
	{
		return ERROR_TCPSOCKET_FAILED_TO_LISTEN;
	}

	if (aAddress != NULL)
	{
		memset ((void *)aAddress, 0x0, sizeof (stSockAddrIn));
		memcpy ((void *)aAddress, (void *)&mAddress, sizeof (stSockAddrIn));
	}

	/*----------------------------------------------------------------*/
	return aResult;
}

int
CRXSocket::Accept (stSockAddrIn * aRemoteAddress,
				   int			* aAddressLength)
{
	int				aResult = -1;

	stSockAddrIn	aAddress;
	socklen_t		aLength = sizeof (aAddress);

	/*----------------------------------------------------------------*/
	if (!IsReady ())		return ERROR_TCPSOCKET_NOT_READY;

	aResult = accept (mSocket, (stSockAddr *) &aAddress, &aLength);
	if (aResult < 0)
	{
		return aResult;
	}

	if (aRemoteAddress != NULL)
	{
		memset ((void *)aRemoteAddress, 0x0, sizeof (stSockAddrIn));
		memcpy ((void *)aRemoteAddress, (void *)&aAddress, sizeof (stSockAddrIn));
	}

	if (aAddressLength != NULL)
		*aAddressLength = aLength;

	/*----------------------------------------------------------------*/
	return aResult;
}

int
CRXSocket::Send (char	* aBuffer,
				 int	aSize)
{
	int aResult = -1;

	/*----------------------------------------------------------------*/
	if (!IsReady ())		return ERROR_TCPSOCKET_NOT_READY;

	aResult = send (mSocket, aBuffer, aSize, 0);
	if (aResult != aSize)
	{
		return ERROR_TCPSOCKET_FAILED_TO_SEND;
	}

	/*----------------------------------------------------------------*/
	return aResult;
}

int
CRXSocket::Receive (char	* aBuffer,
					int		aSize)
{
	int aResult = -1;

	/*----------------------------------------------------------------*/
	if (!IsReady ())		return ERROR_TCPSOCKET_NOT_READY;

	aResult = recv (mSocket, aBuffer, aSize, 0);
	if (aResult < 0)
	{
		return ERROR_TCPSOCKET_FAILED_TO_RECEIVE;
	}

	/*----------------------------------------------------------------*/
	return aResult;
}
